@@ -3,18 +3,19 @@ import warnings
 import torch.nn as nn
 from torch.optim import RMSprop
 
+from torch.utils.data import (random_split,
+                              DataLoader,
+                              Dataset)
+from torch import tensor, Generator, concat
+from torchvision import transforms
+from torch.utils.data import TensorDataset
+
 from torchmetrics import Accuracy
 
 from pytorch_lightning import (LightningModule,
                                LightningDataModule)
 from pytorch_lightning.utilities.distributed import rank_zero_only
-from pytorch_lightning.utilities.seed import seed_everything
-from torch.utils.data import (random_split,
-                              DataLoader,
-                              Dataset)
-from torch import tensor
-from torchvision import transforms
-from torch.utils.data import TensorDataset
+from pytorch_lightning.callbacks import Callback
 
 class SimpleDataModule(LightningDataModule):
 
@@ -45,7 +46,8 @@ class SimpleDataModule(LightningDataModule):
             (self.train_dataset,
              self.validation_dataset) = random_split(train_dataset,
                                                      [ntrain - nvalidation,
-                                                      nvalidation])
+                                                      nvalidation],
+                                                     generator=Generator().manual_seed(seed))
                 
         self.test_dataset = test_dataset
         self.batch_size = batch_size
@@ -54,7 +56,6 @@ class SimpleDataModule(LightningDataModule):
         self.seed = seed
         
     def train_dataloader(self):
-        seed_everything(self.seed, workers=True)
         return DataLoader(self.train_dataset,
                           shuffle=True,
                           batch_size=self.batch_size,
@@ -67,22 +68,19 @@ class SimpleDataModule(LightningDataModule):
     # over minibatch
 
     def val_dataloader(self):
-        seed_everything(self.seed, workers=True)
         return DataLoader(self.validation_dataset,
                           shuffle=False,
-                          batch_size=len(self.validation_dataset),
+                          batch_size=self.batch_size,
                           num_workers=self.num_workers,
                           persistent_workers=self.persistent_workers)
 
     def test_dataloader(self):
-        seed_everything(self.seed, workers=True)
         return DataLoader(self.test_dataset,
-                          batch_size=len(self.test_dataset),
+                          batch_size=self.batch_size,
                           num_workers=self.num_workers,
                           persistent_workers=self.persistent_workers)
 
     def predict_dataloader(self):
-        seed_everything(self.seed, workers=True)
         return DataLoader(self.test_dataset,
                           batch_size=len(self.test_dataset),
                           num_workers=self.num_workers,
@@ -109,7 +107,8 @@ class SimpleDataModule(LightningDataModule):
         train_ds, test_ds, valid_ds = random_split(tensor_ds,
                                                    [npts - test - validation,
                                                     test,
-                                                    validation])
+                                                    validation],
+                                                   generator=Generator().manual_seed(seed))
         if test_as_validation:
             valid_ds = test_ds
             if validation != 0:
@@ -143,7 +142,7 @@ class SimpleModule(LightningModule):
         self.loss = loss or nn.MSELoss()
         optimizer = optimizer or RMSprop(model.parameters())
         self._optimizer = optimizer
-        self._metrics = metrics
+        self.metrics = metrics
         self.on_epoch = on_epoch
         self.pre_process_y_for_metrics = pre_process_y_for_metrics
         
@@ -160,43 +159,18 @@ class SimpleModule(LightningModule):
                  on_step=False)
 
         y_ = self.pre_process_y_for_metrics(y)
-        for _metric in self._metrics.keys():
+        for _metric in self.metrics.keys():
             self.log(f"train_{_metric}",
-                     self._metrics[_metric](preds, y_),
+                     self.metrics[_metric](preds, y_),
                      on_epoch=self.on_epoch)
         return loss
 
     def test_step(self, batch, batch_idx):
         x, y = batch
-        preds = self.forward(x)
-        loss = self.loss(preds, y)
-
-        y_ = self.pre_process_y_for_metrics(y)
-        for _metric in self._metrics.keys():
-            self.log(f"test_{_metric}",
-                     self._metrics[_metric](preds, y_),
-                     on_epoch=self.on_epoch)
-        self.log("test_loss",
-                 loss,
-                 on_epoch=self.on_epoch)
 
     @rank_zero_only
     def validation_step(self, batch, batch_idx):
         x, y = batch
-        preds = self.forward(x)
-        loss = self.loss(preds, y)
-        # convert before computing metrics
-        # needed for BCEWithLogitsLoss -- y is float but
-        # must be int for classification metrics
-
-        y_ = self.pre_process_y_for_metrics(y)
-        for _metric in self._metrics.keys():
-            self.log(f"valid_{_metric}",
-                     self._metrics[_metric](preds, y_),
-                     on_epoch=self.on_epoch)
-        self.log("valid_loss",
-                 loss,
-                 on_epoch=self.on_epoch)
 
     def predict_step(self, batch, batch_idx):
         x, y = batch
@@ -215,9 +189,15 @@ class SimpleModule(LightningModule):
 
     @staticmethod
     def binary_classification(model,
-                              metrics={'accuracy':Accuracy()},
+                              metrics={},
+                              device=None,
                               **kwargs):
         loss = nn.BCEWithLogitsLoss()
+        if 'accuracy' not in metrics:
+            metrics['accuracy'] = Accuracy()
+        if device is not None:
+            for key, metric in metrics:
+                metrics[key] = metric.to(device)
         return SimpleModule(model,
                             loss,
                             metrics=metrics,
@@ -226,11 +206,87 @@ class SimpleModule(LightningModule):
 
     @staticmethod
     def classification(model,
-                       metrics={'accuracy':Accuracy()},
+                       metrics={},
+                       device=None,
                        **kwargs):
         loss = nn.CrossEntropyLoss()
+        if 'accuracy' not in metrics:
+            metrics['accuracy'] = Accuracy()
+        if device is not None:
+            for key, metric in metrics:
+                metrics[key] = metric.to(device)
         return SimpleModule(model,
                             loss,
                             metrics=metrics,
                             **kwargs)
     
+class ErrorTracker(Callback):
+
+    def on_validation_epoch_start(self,
+                                  trainer,
+                                  pl_module):
+        self.val_preds = []
+        self.val_targets = []
+
+    def on_validation_batch_start(self,
+                                  trainer,
+                                  pl_module,
+                                  batch,
+                                  batch_idx,
+                                  dataloader_idx):
+        x, y = batch
+        self.val_preds.append(pl_module.forward(x))
+        self.val_targets.append(y)
+
+    def on_validation_epoch_end(self,
+                                trainer,
+                                pl_module):
+
+        preds = concat(self.val_preds)
+        targets = concat(self.val_targets)
+        targets_ = pl_module.pre_process_y_for_metrics(targets)
+
+        loss = pl_module.loss(preds, targets)
+        pl_module.log("valid_loss",
+                      loss,
+                      on_epoch=pl_module.on_epoch)
+
+        for _metric in pl_module.metrics.keys():
+            pl_module.log(f"valid_{_metric}",
+                          pl_module.metrics[_metric](preds, targets_),
+                          on_epoch=pl_module.on_epoch)
+
+    def on_test_epoch_start(self,
+                            trainer,
+                            pl_module):
+        self.test_preds = []
+        self.test_targets = []
+
+    def on_test_batch_start(self,
+                            trainer,
+                            pl_module,
+                            batch,
+                            batch_idx,
+                            dataloader_idx):
+        x, y = batch
+        self.test_preds.append(pl_module.forward(x))
+        self.test_targets.append(y)
+
+    def on_test_epoch_end(self,
+                          trainer,
+                          pl_module):
+
+        preds = concat(self.test_preds)
+        targets = concat(self.test_targets)
+        targets_ = pl_module.pre_process_y_for_metrics(targets)
+
+        loss = pl_module.loss(preds, targets)
+        pl_module.log("test_loss",
+                      loss,
+                      on_epoch=pl_module.on_epoch)
+
+        for _metric in pl_module.metrics.keys():
+            pl_module.log(f"test_{_metric}",
+                          pl_module.metrics[_metric](preds, targets_),
+                          on_epoch=pl_module.on_epoch)
+
